/**
 * @file dmxstr.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ----------------------------------------------------------------------------
 * (c) Florian Schmitt / 2012-2013
 * License GPLv3+: GNU GPL Version 3 
 * or higher <http://gnu.org/licenses/gpl.html>
 * This code comes with ABSOLUTELY NO WARRANTY. This is free software, and 
 * you are welcome to redistribute it under certain conditions.
 * ----------------------------------------------------------------------------
 * Special String Commands for lindmx
**/
#include "dmxstr.h"
#include <stdio.h>
#include <stdlib.h>
//#include <string.h>


/**
 * Check if a character is whitespace
 * @return >=1 if so, 0 if not
**/
int dmxstr_isspace(char c)
{
	if( (c==' ')  ||
	    (c=='\t') )
	{
		return 1;
	}
	return 0;
}



/**
 * Check if a character is an end limiter (newline, linefeed, null)
 * @return >=1 if so, 0 if not
**/
int dmxstr_isend(char c)
{
	if( (c=='\n')  ||
	    (c=='\r')  ||
	    (c=='\0') )
	{
		return 1;
	}
	return 0;
}



/**
 * Check if a character is numeric
 * @param ishex  if not zero, a-f and A-F is allowed
 * @return >=1 if so, 0 if not
**/
int dmxstr_isnumeral(char c, int ishex)
{
	if( (c >= '0') && (c <= '9') )
		return 1;
	if( ishex && (c >= 'a') && (c <= 'f') )
		return 1;
	if( ishex && (c >= 'A') && (c <= 'F') )
		return 1;
	return 0;
}



/**
 * Check if a character is a number (with range-end)
 * format: [+/-][0x]{0,1,2,3,4,5,6,7,8,9}*
 * @return >=1 if so, 0 if not
**/
int dmxstr_isnumber_ext(const char *c, int allowrange)
{
	/* states [0=empty,1=presign,2=type,3=number] */
	int state = 0;
	int n = 0;
	int hex = 0;
	/* main loop - until space is reached */
	while( 1 )
	{
		/* break if whitespace/end */
		if( dmxstr_isspace(c[n]) ||
		    ( (c[n] == DMXSTR_RANGE_SEPERATOR ) && (state > 0) && allowrange ) ||
		    dmxstr_isend(c[n])      )
			break;
		/* state 0 - empty */
		if(state == 0)
		{
			/* presign */
			if( (c[n] == '+') || (c[n] == '-') )
				{ state = 1; }
			/* hex number */
			else if( (c[n] == '0') && (c[n+1] == 'x') )
				{ state = 2; hex = 1; n++; }
			else if( (c[n] == '0') && (c[n+1] == 'X') )
				{ state = 2; n++; }
			/* number input */
			else if( (c[n] >= 48) && (c[n] <= 57) )
				{ state = 3; }
			/* invalid */
			else
				{ return 0; }
		}
		/* state 1 - presign given */
		else if(state == 1)
		{
			/* hex number */
			if( (c[n] == '0') && (c[n+1] == 'x') )
				{ state = 2; n++; }
			else if( (c[n] == '0') && (c[n+1] == 'X') )
				{ state = 2; hex = 1; n++; }
			/* number input */
			else if( (c[n] >= 48) && (c[n] <= 57) )
				{ state = 3; }
			/* invalid */
			else
				{ return 0; }
		}
		/* state 2 - type given */
		else if(state == 2)
		{
			/* number input */
			if( dmxstr_isnumeral(c[n], hex) )
				state = 3;
			else
				return 0;
		}
		/* state 3 - numbers given */
		else if(state == 3)
		{
			/* number input */
			if( !(dmxstr_isnumeral(c[n], hex)) )
				return 0;
		}
		/* next */
		n++;
	}
	/* it's a valid number if state 2 is reached */
	if(state == 3) return 1;
	return 0;
}



/**
 * SEE: dmxstr_isnumber_ext
**/
int dmxstr_isnumber(const char *c)
{
	return dmxstr_isnumber_ext(c, 0);
}



/**
 * Ignores leading whitespace
 * returns a string beginning without whitespace
**/
const char* dmxstr_remove_whitespace(const char *c)
{
	unsigned int n = 0;
	while( dmxstr_isspace(c[n]) )
		n++;
	return (const char *)( c + n );
}



/**
 * Check if string is a comment or an empty line
**/
int dmxstr_is_comment(const char *c)
{
	c = dmxstr_remove_whitespace(c);
	/* empty line */
	if( dmxstr_isend(c[0]) ) return 1;
	/* comment */
	if( c[0] == '#' ) return 1;
	return 0;
}



/**
 * Checks if the string is of a given command
 * removes leading whitespace before automatically
 * String can be spelled in upper- or lowercase
 * @param command   COMMAND MUST BE LOWERCASE !!! !!! !!!
 * @return 0 if not this command, otherwise address of next parameter
**/
const char* dmxstr_chkcmd(const char *string, const char *command)
{
	unsigned int n = 0;
	/* skip leading whitespace */
	string = dmxstr_remove_whitespace(string);
	/* loop out */
	while( !dmxstr_isend(command[n]) )
	{
		/* if not equal */
		if( (string[n] != command[n]) && (string[n] != (command[n]-32)) )
			return 0;
		n++;
	}
	/* whitespace / command end */
	if( dmxstr_isspace(string[n]) || dmxstr_isend(string[n]))
		return (const char*)(string + n);
	/* string is longer */
	return 0;
}



/**
 * Copy next parameter as string into buffer
 * @return 0 if no parameter read, otherwise address of next parameter
**/
const char* dmxstr_get_next_param_str(const char *string, char *buffer, int buflen)
{
	unsigned int n = 0;
	/* skip leading whitespace */
	string = dmxstr_remove_whitespace(string);
	/* loop out */
	while( 1 )
	{
		/* check if buflen exceeded */
		if(n >= (buflen-1)) break;
		/* param or command ends */
		if( dmxstr_isspace(string[n]) || dmxstr_isend(string[n]) )
			break;
		buffer[n] = string[n];
		n++;
	}
	/* Set buffer end */
	buffer[n] = '\0';
	/* command's line ends */
	if( n == 0 )
		return 0;
	return (const char*)(string + n);
}



/**
 * Get next parameter as integer
 * format: decimal, leading '0x' for hex, +/- allowed
 * @return 0 if no parameter read, otherwise address of next parameter
**/
const char* dmxstr_get_next_param_int(const char *string, int *number)
{
	char *next = 0;
	/* skip leading whitespace */
	string = dmxstr_remove_whitespace(string);
	/* check if parameter is a correct number */
	if( !dmxstr_isnumber(string) )
		return 0;
	/* get number / strtol skips whitespace itself */
	*number = strtol(string, &next, 0);
	/* return position behind the number */
	return next;
}



/**
 * Get next parameter as integer range
 * format: number-number
 * @return 0 if no parameter read, otherwise address of next parameter
**/
const char* dmxstr_get_next_param_range(const char *string, int *from, int *to)
{
	char *next = 0;
	/* skip leading whitespace */
	string = dmxstr_remove_whitespace(string);
	/* check if parameter is a correct number (range) */
	if( !dmxstr_isnumber_ext(string, 1) )
		return 0;
	/* get number / strtol skips whitespace itself */
	*from = strtol(string, &next, 0);
	/* check if range is given or single number */
	if(next[0] != DMXSTR_RANGE_SEPERATOR)
	{
		*to = *from;
		return next;
	}
	/* continue with next number */
	next++;
	if( !dmxstr_isnumber(next) )
		return 0;
	/* get number / strtol skips whitespace itself */
	*to = strtol(next, &next, 0);
	/* toogle if wrong direction */
	if(*from > *to)
	{
		int tmp = *from;
		*from = *to;
		*to = tmp;
	}
	/* return position behind the number */
	return next;
}



/**
 * Translate step-length to a string in measure-format
 * format: number-number
 * @return 1 on success
**/
int dmxstr_translate_steplen(char *strbuf, long int steplen)
{
	if(strbuf == 0) return 0;
	if(steplen > 0) sprintf(strbuf, "%ld",   steplen);
	else            sprintf(strbuf, "1/%ld", (-1)*(steplen));
	return 1;
}



/**
 * Translate step-length to a string in measure-format
 * format: number-number
 * @return 1 on success, 0 on buffer overflow or string error
**/
int dmxstr_get_script_filename(char *filename, int buflen, const char *scriptdir, const char *cmd)
{
	int n, i;
	if(cmd == 0) return 0;
	for(n=0; scriptdir[n]!='\0'; n++)
	{
		if(n>=(buflen-1))
		{
			filename[0] = '\0';
			return 0;
		}
		filename[n] = scriptdir[n];
	}
	/* Add / (directory seperator) */
	filename[n++] = DMXSTR_DIR_SEPERATOR;
	/* add filename */
	for(i=0; ((cmd[i]!='\0') && (cmd[i]!='\t') && (cmd[i]!=' ')); i++)
	{
		if(n>=(buflen-1))
		{
			filename[0] = '\0';
			return 0;
		}
		filename[n] = cmd[i];
		n = n + 1;
	}
	filename[n] = '\0';
	return 1;
}



/**
 * Translate step-length to a string in measure-format
 * format: number-number
 * @return string length of target
**/
int dmxstr_parse_value(char *target, const char *cmd, midib data)
{
	int t_cnt, c_cnt, n_cnt;
	unsigned int scale;
	char numbuf[50];
	t_cnt = 0;
	c_cnt = 0;
	/* main loop */
	while(1)
	{
		/* END condition */
		if(cmd[c_cnt] == '\0')
		{
			target[t_cnt++] = cmd[c_cnt++];
			break;
		}
		/* ON/OFF switch */
		else if(cmd[c_cnt] == '$')
		{
			c_cnt++;
			if(data > (127/2))
			{
				target[t_cnt++] = 'O';
				target[t_cnt++] = 'N';
			}
			else
			{
				target[t_cnt++] = 'O';
				target[t_cnt++] = 'F';
				target[t_cnt++] = 'F';
			}
		}
		/* insert value */
		else if(cmd[c_cnt] == '%')
		{
			c_cnt++;
			/* get number behind % into 'scale' */
			n_cnt = 0;
			while( (cmd[c_cnt] >= '0') && (cmd[c_cnt] <= '9') && (n_cnt < 49) )
				numbuf[n_cnt++] = cmd[c_cnt++];
			numbuf[n_cnt++] = '\0';
			scale = atoi(numbuf);
			/* write scaled data back to numbuf */
			if(scale == 0) snprintf(numbuf, 50, "%d", ((255*data)/127));
			else           snprintf(numbuf, 50, "%d", ((scale*data)/127));
			/* copy numbuf into target */
			n_cnt = 0;
			while( (numbuf[n_cnt] != '\0') )
				target[t_cnt++] = numbuf[n_cnt++];
		}
		/* just text */
		else
		{
			target[t_cnt++] = cmd[c_cnt++];
		}
	}
	return t_cnt;
}
