/**
 * @file modules/dmx4all.c
 * @author Florian Schmitt <florian.schmitt@qikgrp.com>
 * ---------------------------------------------------------
 * (c) Florian Schmitt / Nov-2012 / All rights reserved. 
 * ---------------------------------------------------------
 * Implements DMX4ALL interface
**/
#include "dmx4all.h"
#include <syslog.h>
#include <stdlib.h>
#include <string.h>
#include <ftdi.h>

int    channels;
struct ftdi_context interface;



/**
 * getFormatedStr()
 * Creates a command-string to send a value for a given channel
 * @param channel  The channel to set
 * @param value    The value to set to
 * @param buf      Must be min. 9 chars long! -> Here the data is written
 * @return Should be 8, 0=NO DATA, -1=FAIL (length of written data)
**/
int getFormatedStr(int channel, int value, unsigned char *buf)
{
	/* char counter */
	int cnt = 0;
	/* check if channel&value is valid */
	buf[0] = '\0';
	if( (channel < 0) || (channel > 511) ) return -1;
	if( (value   < 0) || (value   > 255) ) return -1;
	/* channel */
	buf[cnt++] = 'C';
	buf[cnt++] = '0'+(channel/100)%10;
	buf[cnt++] = '0'+((channel/10)%10);
	buf[cnt++] = '0'+(channel%10);
	/* value */
	buf[cnt++] = 'L';
	buf[cnt++] = '0'+(value/100)%10;
	buf[cnt++] = '0'+((value/10)%10);
	buf[cnt++] = '0'+(value%10);
	/* set end and return length */
	buf[cnt+1] = '\0';
	return cnt;
}



/**
 * getFormatedStrChannels()
 * Creates a command-string to set the number of channels to work with
 * @param channel  The channel to set
 * @param buf      Must be min. 9 chars long! -> Here the data is written
 * @return Should be 8, 0=NO DATA, -1=FAIL (length of written data)
**/
int getFormatedStrChannels(int channels, unsigned char *buf)
{
	/* char counter */
	int cnt = 0;
	/* check if channel&value is valid */
	buf[0] = '\0';
	if( (channels < 0) || (channels > 511) ) return -1;
	/* channel cnt */
	buf[cnt++] = 'N';
	buf[cnt++] = '0'+(channels/100)%10;
	buf[cnt++] = '0'+((channels/10)%10);
	buf[cnt++] = '0'+(channels%10);
	/* set end and return length */
	buf[cnt+1] = '\0';
	return cnt;
}


/**
 * dmx4all_init()
 * Init DMX4ALL Micro USB Interface
 * @param maxchannel    The highest channel number created
 * @return 1 on success, 0 on failure
**/
int dmx4all_init(int maxchannel)
{
	unsigned char cbuf[10];
	int sl, written;
	/* set channel count */
	channels = maxchannel;
	/* Init FTDI */
	if(ftdi_init(&interface) < 0)
	{
     	syslog(LOG_ERR, "FTDI can not be initialized!");
		return 0;
	}
	/* Open FTDI-USB-Interface */
	if(ftdi_usb_open(&interface, 0x0403, 0xc850) < 0) 
	{
     	syslog(LOG_ERR, "DMX4ALL Micro USB Interface can not be found!");
		return 0;
	}
	/* Set FTDI-Baudrate 3840 */
	ftdi_set_baudrate(&interface, 38400);
	/* Set channel count */
	sl = getFormatedStrChannels(channels, cbuf);
	if(sl<=0) 
	{
		ftdi_usb_close(&interface);
		ftdi_deinit(&interface);
     	syslog(LOG_ERR, "Error on string operation: getFormatedStrChannels");
		return 0;
	}
	written  = ftdi_write_data(&interface, cbuf, sl);
	if(written<sl) 
	{
		ftdi_usb_close(&interface);
		ftdi_deinit(&interface);
     	syslog(LOG_ERR, "Error writing data to DMX4ALL Micro USB Interface");
		return 0;
	}
	/* Recv */
	ftdi_read_data(&interface, cbuf, 12);
	/* success */
	return 1;
}



/**
 * dmx4all_destroy()
 * Clean up interface
**/
void dmx4all_destroy()
{
	/* Close & Deinit */
	ftdi_usb_close(&interface);
	ftdi_deinit(&interface);
	channels = 0;
}



/**
 * dmx4all_set()
 * Sets a channel to a given value
 * @param channel  Channel to set
 * @param value    Value to set to
 * @return 1 on success, 0 on failure
**/
int dmx4all_set(int channel, int value)
{
	/* Buffer&Vars */
	unsigned char buf[16];
	int sl, written;
	/* Get Formated String */
	sl = getFormatedStr(channel, value, buf);
	if(sl<=0) return 0;
	/* Send */
	written = ftdi_write_data(&interface, buf, sl);
	if(written<sl)
	{
     	syslog(LOG_ERR, "Setting a channel failed: Error writing data to DMX4ALL Micro USB Interface");
		return 0;
	}
	/* Recv */
	ftdi_read_data(&interface, buf, 12);
	return 1;
}



/**
 * dmx4all_set_all()
 * Sets all channels to a given value
 * @param value    Value to set all channels to
 * @return 1 on success, 0 on failure
**/
int dmx4all_set_all(int value)
{
	int ret, n;
	ret = 1;
	for(n=0; n<=channels; n++)
	{
		if( dmx4all_set(n, value) <= 0 ) ret = 0;
	}
	return ret;
}



/**
 * dmx4all_reset_channels()
 * Resets all channels to zero
 * @param maxchannel    The highest channel number created
 * @return 1 on success, 0 on failure
**/
int dmx4all_reset_channels()
{
	return dmx4all_set_all(0);
}
